@@ -5,18 +5,20 @@ from flask.json import JSONEncoder
 
 from sqlalchemy.orm.collections import InstrumentedList
 
-from sqlalchemy_plus import serialize
-from sqlalchemy_plus import iserialize
+from sqlalchemy_abc import serialize
+from sqlalchemy_abc import iserialize
+from handyhttp import HTTPException
 
 
 class ModelEncoder(JSONEncoder):
     def default(self, data):
+        if isinstance(data, HTTPException):
+            return data.pack()
         if isinstance(data, InstrumentedList):
-            collection = serialize(data)
+            return serialize(data)
         else:
             data = iserialize(data)
         return data
-        return JSONEncoder.default(self, data)
 
 
 class JSONResponse(Response):
